import matplotlib.pyplot as plt
import matplotlib.image as mpimg

from agilent_format import agilentMosaicIFG

PLOTS = True
ATTRS = False

filename = ["var/2018-01-02 Small Test File/5_mosaic_noimage_agg1024/5_mosaic_agg1024.dmt"]

for f in filename:
    aifg = agilentMosaicIFG(f)
    print(aifg.data.shape)

    if PLOTS:
        img = aifg.data.sum(axis=2)
        print(img.shape)
        imgplot = plt.imshow(img, origin='lower')
        plt.show()
        plt.plot(range(0, aifg.data.shape[2]), aifg.data[0,0,:])
        plt.plot(range(0, aifg.data.shape[2]), aifg.data[2,2,:])
        plt.plot(range(0, aifg.data.shape[2]), aifg.data[0,3,:])
        plt.show()

    assert aifg.data.shape == (8, 4, 311)
    assert aifg.info['Npts'] == 311
    assert aifg.info['StartPt'] == -68
    assert aifg.info['PtSep'] == float(0.00012659827227975054)
    assert aifg.info['Rapid Stingray']['Effective Laser Wavenumber'] == "15798.0039"
    assert aifg.info['Rapid Stingray']['Resolution'] == "32"
    try:
        assert aifg.info['Rapid Stingray']['Symmetry'] == "ASYM"
    except KeyError:
        print("No \'Symmetry\' key in " + aifg.filename)
    try:
        assert aifg.info['Rapid Stingray']['Under Sampling Ratio'] == "4"
    except KeyError:
        print("No \'Under Sampling Ratio\' key in " + aifg.filename)
    assert aifg.info['PixelAggregationSize'] == 32


if ATTRS:
    for k,v in aifg.info.items():
        if k == "Rapid Stingray":
            for k, v, in v.items():
                print(k,v, type(v))
        else:
            print(k,v, type(v))


import shutil
import tempfile
import unittest
from pathlib import Path

from numpy import float64, isnan

from agilent_format import agilentMosaic

DMT = Path(__file__).parent.parent.joinpath("datasets/5_mosaic_agg1024.dmt")


class TestMosaic(unittest.TestCase):

    def test_load_mosaic(self):
        ai = agilentMosaic(DMT, MAT=False)
        self.assertEqual(ai.filename, str(DMT))
        # Check parameters
        Npts = ai.info['Npts']
        self.assertEqual(Npts, 9)
        self.assertEqual(Npts, len(ai.wavenumbers))
        self.assertEqual(ai.wavenumbers[0], ai.info['StartPt'] * ai.info['PtSep'])
        self.assertEqual(ai.wavenumbers[-1],
                         (ai.info['StartPt'] + Npts - 1) * ai.info['PtSep'])
        self.assertEqual(ai.data.shape, (8, 4, Npts))
        self.assertEqual(ai.info['FPA Pixel Size'], 5.5)
        self.assertEqual(ai.info['PixelAggregationSize'], 32)
        # Confirm image orientation
        self.assertAlmostEqual(ai.data[0, 1, 1], 1.14792180)
        self.assertAlmostEqual(ai.data[0, 2, 2], 1.14063489)
        self.assertAlmostEqual(ai.data[1, 2, 3], 0.28298783)

    def test_load_mosaic_MAT(self):
        ai = agilentMosaic(DMT, MAT=True)
        # Confirm image orientation
        self.assertAlmostEqual(ai.data[7, 1, 1], 1.14792180)
        self.assertAlmostEqual(ai.data[7, 2, 2], 1.14063489)
        self.assertAlmostEqual(ai.data[6, 2, 3], 0.28298783)

    def test_load_mosaic_64(self):
        ai = agilentMosaic(DMT, MAT=False, dtype=float64)
        self.assertAlmostEqual(ai.data[1, 2, 3], 0.28298783)

    def test_load_mosaic_vis(self):
        ai = agilentMosaic(DMT, MAT=False)
        self.assertEqual(len(ai.vis), 2)
        self.assertDictEqual(ai.vis[0],
                             {'name': "IR Cutout",
                              'image_ref': Path(DMT).parent.joinpath("IrCutout.bmp"),
                              'pos_x': 0,
                              'pos_y': 0,
                              'img_size_x': 701.0,
                              'img_size_y': 1444.0,})
        self.assertDictEqual(ai.vis[1],
                             {'name': "Entire Image",
                              'image_ref': Path(DMT).parent.joinpath("VisMosaicCollectImages_Thumbnail.bmp"),
                              'pos_x': -204.400785854617,
                              'pos_y': -201.8752228163994,
                              'img_size_x': 1530.0,
                              'img_size_y': 1688.0,})

    def test_load_partial_mosaic(self):
        """Test a (synthetic) incomplete mosaic"""
        with tempfile.TemporaryDirectory() as dir_name:
            dest = Path(dir_name)
            dmt = Path(DMT)
            shutil.copyfile(dmt, dest.joinpath(dmt.name))
            dmd0 = dmt.parent.joinpath("5_Mosaic_agg1024_0000_0000.dmd")
            shutil.copyfile(dmd0, dest.joinpath(dmd0.name))
            dmd1 = dmt.parent.joinpath("5_Mosaic_agg1024_0000_0001.dmd")
            shutil.copyfile(dmd1, dest.joinpath(dmd1.name))
            dmd2 = "5_Mosaic_agg1024_0001_0000.dmd"
            shutil.copyfile(dmd0, dest.joinpath(dmd2))

            ai = agilentMosaic(Path(dir_name, dmt.name), MAT=False)
            # Confirm image orientation (completed data)
            self.assertAlmostEqual(ai.data[0, 1, 1], 1.14792180)
            self.assertAlmostEqual(ai.data[0, 2, 2], 1.14063489)
            self.assertAlmostEqual(ai.data[1, 2, 3], 0.28298783)
            # Confirm empty region (4th quadrant) is NaNs
            self.assertTrue(isnan(ai.data[0, 4, 5]))



import unittest
from pathlib import Path

from agilent_format import agilentImageIFG, agilentImage

SEQ = Path(__file__).parent.parent.joinpath("datasets/4_noimage_agg256.seq")
SEQR = Path(__file__).parent.parent.joinpath("datasets/background_agg256.seq")
DAT = Path(__file__).parent.parent.joinpath("datasets/background_agg256.dat")

class TestImageIFG(unittest.TestCase):

    def shared_info(self, aifg):
        self.assertEqual(aifg.data.shape, (8, 8, 311))
        self.assertEqual(aifg.info['Npts'], 311)
        self.assertEqual(aifg.info['StartPt'], -68)
        self.assertEqual(aifg.info['PtSep'], float(0.00012659827227975054))
        self.assertEqual(aifg.info['Effective Laser Wavenumber'], 15798.0039)
        self.assertEqual(aifg.info['Resolution'], 32)
        self.assertEqual(aifg.info['Symmetry'], "ASYM")
        self.assertEqual(aifg.info['Under Sampling Ratio'], 4)
        self.assertEqual(aifg.info['PixelAggregationSize'], 16)

    def test_load_ifg_sample(self):
        aifg = agilentImageIFG(SEQ, MAT=False)
        self.shared_info(aifg)
        # Confirm image orientation
        self.assertAlmostEqual(aifg.data[1, 1, 0], 0.64558595)
        self.assertAlmostEqual(aifg.data[2, 2, 0], 0.5792696)

    def test_load_ifg_sample_MAT(self):
        aifg = agilentImageIFG(SEQ, MAT=True)
        self.shared_info(aifg)
        # Confirm image orientation
        self.assertAlmostEqual(aifg.data[1, 1, 0], 0.39676425)
        self.assertAlmostEqual(aifg.data[2, 2, 0], 0.8491539)

    def test_load_ifg_ref(self):
        aifg = agilentImageIFG(SEQR, MAT=False)
        self.shared_info(aifg)
        # Confirm image orientation
        self.assertAlmostEqual(aifg.data[1, 1, 0], 0.97700727)
        self.assertAlmostEqual(aifg.data[2, 2, 0], 1.0310643)

    def test_ifg_processed_wn(self):
        aifg = agilentImageIFG(SEQ, MAT=False)
        wn_ifg = aifg.info['wavenumbers']
        ai = agilentImage(DAT, MAT=False)
        self.assertEqual(ai.info['wavenumbers'], wn_ifg)



import matplotlib.pyplot as plt
import matplotlib.image as mpimg

from agilent_format import agilentImageIFG

PLOTS = False
ATTRS = True

filename = ["var/2018-01-02 Small Test File/4_noimage_agg256.bsp"]
# filename += ["var/2018-01-02 Small Test File/background_agg256.bsp"]

for f in filename:
    aifg = agilentImageIFG(f)
    print(aifg.data.shape)

    if PLOTS:
        img = aifg.data.sum(axis=2)
        print(img.shape)
        imgplot = plt.imshow(img, origin='lower')
        plt.show()
        plt.plot(range(0, aifg.data.shape[2]), aifg.data[0,0,:])
        plt.plot(range(0, aifg.data.shape[2]), aifg.data[2,2,:])
        plt.plot(range(0, aifg.data.shape[2]), aifg.data[0,3,:])
        plt.show()

    assert aifg.data.shape == (8, 8, 311)
    assert aifg.info['Npts'] == 311
    assert aifg.info['StartPt'] == -68
    assert aifg.info['PtSep'] == float(0.00012659827227975054)
    assert aifg.info['Effective Laser Wavenumber'] == 15798.0039
    assert aifg.info['Resolution'] == 32
    assert aifg.info['Symmetry'] == "ASYM"
    assert aifg.info['Under Sampling Ratio'] == 4
    assert aifg.info['PixelAggregationSize'] == 16

if ATTRS:
    for k,v in aifg.info.items():
        if k == "Rapid Stingray":
            for k, v, in v.items():
                print(k, "|", v, "|", type(v))
        else:
            print(k, "|", v, "|", type(v))


import struct

from agilent_format import _get_params

filename = "var/2017-11-10 4X-25X/2017-11-10 4x-25x.dmt"
#filename = "var/2017-11-10 4X-25X/2017-11-10 4x-25x.bsp"
#filename = "var/2018-01-02 Small Test File/4_noimage_agg256.bsp"
#filename = "var/2018-01-02 Small Test File/background_agg256.bsp"
#filename = "var/2018-01-02 Small Test File/background_agg1024.bsp"
filename = "C:\\Users\\reads\\tmp\\aff-testdata\\2017-11-24 USAF after FPA pumpdown\\USAF 25X Mosaic\\usaf 25x mosaic.bsp"

with open(filename, 'rb') as f:
    d = _get_params(f)

for k, v in d.items():
    try:
        for k2, v2 in v.items():
            print("{0} : {1}".format(k2, v2))
    except AttributeError:
        print("{0} : {1}".format(k, v))



import pathlib
import tempfile
import unittest

import agilent_format


class TestFilenames(unittest.TestCase):

    def setUp(self) -> None:
        self._temp_dir = tempfile.TemporaryDirectory()
        self._temp_path = pathlib.Path(self._temp_dir.name)

    def tearDown(self) -> None:
        self._temp_dir.cleanup()

    def add_files_to_dir(self, filenames):
        p = self._temp_path
        for fn in filenames:
            p.joinpath(p, fn).touch()

    def test_dmt_DMD(self):
        filenames = ["ab9.dmt",
                     "AB9.bsp",
                     "AB9.dat",
                     "AB9.dms",
                     "AB9.seq",
                     "AB9_0000_0000.dmd",
                     "AB9_0000_0000.drd",
                     ]
        self.add_files_to_dir(filenames)
        dmt = self._temp_path.joinpath(filenames[0])
        self.assertEqual(agilent_format.base_data_path(dmt).name, "AB9")
        self.assertEqual(agilent_format.check_files(dmt, ['.dmt', '.dmd']).name, "AB9")

    def test_dmt_DMD2(self):
        filenames = ["ab9.dmt",
                     "ab9.bsp",
                     "AB9.dat",
                     "AB9.dms",
                     "AB9.seq",
                     "AB9_0000_0000.dmd",
                     "AB9_0000_0000.drd",
                     ]
        self.add_files_to_dir(filenames)
        dmt = self._temp_path.joinpath(filenames[0])
        self.assertEqual(agilent_format.base_data_path(dmt).name, "AB9")
        self.assertEqual(agilent_format.check_files(dmt, ['.dmt', '.dmd']).name, "AB9")

    def test_DMT_DMD(self):
        filenames = ["AB9.dmt",
                     "ab9.bsp",
                     "ab9.dat",
                     "ab9.dms",
                     "ab9.seq",
                     "AB9_0000_0000.dmd",
                     "AB9_0000_0000.drd",
                     ]
        self.add_files_to_dir(filenames)
        dmt = self._temp_path.joinpath(filenames[0])
        self.assertEqual(agilent_format.base_data_path(dmt).name, "AB9")
        self.assertEqual(agilent_format.check_files(dmt, ['.dmt', '.dmd']).name, "AB9")

    def test_dmt_DRD(self):
        filenames = ["ab9.dmt",
                     "AB9.bsp",
                     "AB9.dat",
                     "AB9.dms",
                     "AB9.seq",
                     "AB9_0000_0000.dmd",
                     "AB9_0000_0000.drd",
                     ]
        self.add_files_to_dir(filenames)
        dmt = self._temp_path.joinpath(filenames[0])
        self.assertEqual(agilent_format.base_data_path(dmt).name, "AB9")
        self.assertEqual(agilent_format.check_files(dmt, ['.dmt', '.drd']).name, "AB9")

    def test_dmt_dmd(self):
        filenames = ["ab9.dmt",
                     "ab9.bsp",
                     "ab9.dat",
                     "ab9.dms",
                     "ab9.seq",
                     "ab9_0000_0000.dmd",
                     "ab9_0000_0000.drd",
                     ]
        self.add_files_to_dir(filenames)
        dmt = self._temp_path.joinpath(filenames[0])
        self.assertEqual(agilent_format.base_data_path(dmt).name, "ab9")
        self.assertEqual(agilent_format.check_files(dmt, ['.dmt', '.dmd']).name, "ab9")

    def test_dmt_drd(self):
        filenames = ["ab9.dmt",
                     "ab9.bsp",
                     "ab9.dat",
                     "ab9.dms",
                     "ab9.seq",
                     "ab9_0000_0000.dmd",
                     "ab9_0000_0000.drd",
                     ]
        self.add_files_to_dir(filenames)
        dmt = self._temp_path.joinpath(filenames[0])
        self.assertEqual(agilent_format.base_data_path(dmt).name, "ab9")
        self.assertEqual(agilent_format.check_files(dmt, ['.dmt', '.drd']).name, "ab9")

    def test_dmt_DMD_reprocessed(self):
        filenames = ["ab9.dmt",
                     "ab9.bsp",
                     "ab9.dat",
                     "ab9.dms",
                     "ab9.seq",
                     "AB9_0000_0000.dmd",
                     "AB9_0000_0000.drd",
                     ]
        self.add_files_to_dir(filenames)
        dmt = self._temp_path.joinpath(filenames[0])
        self.assertEqual(agilent_format.base_data_path(dmt).name, "AB9")
        self.assertEqual(agilent_format.check_files(dmt, ['.dmt', '.dmd']).name, "AB9")

    def test_dat_bsp(self):
        filenames = ["ab9.dat",
                     "ab9.bsp",
                     "ab9.seq",
                     ]
        self.add_files_to_dir(filenames)
        dmt = self._temp_path.joinpath(filenames[0])
        self.assertEqual(agilent_format.base_data_path(dmt).name, "ab9.dat")
        self.assertEqual(agilent_format.check_files(dmt, ['.dat', '.bsp']).name, "ab9.dat")

    def test_DAT_bsp(self):
        filenames = ["AB9.dat",
                     "ab9.bsp",
                     "AB9.seq",
                     ]
        self.add_files_to_dir(filenames)
        dmt = self._temp_path.joinpath(filenames[0])
        self.assertEqual(agilent_format.base_data_path(dmt).name, "AB9.dat")
        self.assertEqual(agilent_format.check_files(dmt, ['.dat', '.bsp']).name, "AB9.dat")

    def test_seq_bsp(self):
        filenames = ["ab9.seq",
                     "ab9.dat",
                     "ab9.bsp",
                     ]
        self.add_files_to_dir(filenames)
        dmt = self._temp_path.joinpath(filenames[0])
        self.assertEqual(agilent_format.base_data_path(dmt).name, "ab9.seq")
        self.assertEqual(agilent_format.check_files(dmt, ['.seq', '.bsp']).name, "ab9.seq")

    def test_SEQ_bsp(self):
        filenames = ["AB9.seq",
                     "ab9.bsp",
                     "AB9.dat",
                     ]
        self.add_files_to_dir(filenames)
        dmt = self._temp_path.joinpath(filenames[0])
        self.assertEqual(agilent_format.base_data_path(dmt).name, "AB9.seq")
        self.assertEqual(agilent_format.check_files(dmt, ['.dat', '.bsp']).name, "AB9.seq")


import unittest
from pathlib import Path

from agilent_format import agilentImage

DAT = Path(__file__).parent.parent.joinpath("datasets/4_noimage_agg256.dat")

class TestImage(unittest.TestCase):

    def test_load_image(self):
        ai = agilentImage(DAT, MAT=False)
        # Check parameters
        Npts = ai.info['Npts']
        self.assertEqual(Npts, 9)
        self.assertEqual(Npts, len(ai.wavenumbers))
        self.assertEqual(ai.wavenumbers[0], ai.info['StartPt'] * ai.info['PtSep'])
        self.assertEqual(ai.wavenumbers[-1],
                         (ai.info['StartPt'] + Npts - 1) * ai.info['PtSep'])
        self.assertEqual(ai.data.shape, (8, 8, Npts))
        self.assertEqual(ai.info['FPA Pixel Size'], 5.5)
        self.assertEqual(ai.info['PixelAggregationSize'], 16)
        self.assertEqual(ai.width, ai.data.shape[0])
        self.assertEqual(ai.height, ai.data.shape[1])
        f_bsp = str(DAT.with_suffix(".bsp"))
        self.assertEqual(ai.filename, f_bsp)
        self.assertEqual(ai.acqdate, "Tuesday, January 02, 2018 14:01:52")
        # Confirm image orientation
        self.assertAlmostEqual(ai.data[0, 1, 1], 1.27181053)
        self.assertAlmostEqual(ai.data[0, 2, 2], 1.27506005)
        self.assertAlmostEqual(ai.data[1, 2, 3], 0.30882764)

    def test_load_image_MAT(self):
        ai = agilentImage(DAT, MAT=True)
        # Confirm image orientation
        self.assertAlmostEqual(ai.data[7, 1, 1], 1.27181053)
        self.assertAlmostEqual(ai.data[7, 2, 2], 1.27506005)
        self.assertAlmostEqual(ai.data[6, 2, 3], 0.30882764)


import unittest
from pathlib import Path

from numpy import float64

from agilent_format import agilentMosaicIFG

DMT = Path(__file__).parent.parent.joinpath("datasets/5_mosaic_agg1024.dmt")


class TestMosaicIFG(unittest.TestCase):

    def test_load_ifg_mosaic(self):
        aifg = agilentMosaicIFG(DMT, MAT=False)
        self.assertEqual(aifg.filename, str(DMT))
        # Check parameters
        assert aifg.data.shape == (8, 4, 311)
        assert aifg.info['Npts'] == 311
        assert aifg.info['StartPt'] == -68
        assert aifg.info['PtSep'] == float(0.00012659827227975054)
        assert aifg.info['Effective Laser Wavenumber'] == 15798.0039
        assert aifg.info['Resolution'] == 32
        assert aifg.info['Symmetry'] == "ASYM"
        assert aifg.info['Under Sampling Ratio'] == 4
        assert aifg.info['PixelAggregationSize'] == 32
        # Confirm image orientation
        self.assertAlmostEqual(aifg.data[5, 1, 0], 0.7116039)
        self.assertAlmostEqual(aifg.data[6, 2, 0], 0.48532167)

    def test_load_ifg_mosaic_MAT(self):
        aifg = agilentMosaicIFG(DMT, MAT=True)
        # Confirm image orientation
        self.assertAlmostEqual(aifg.data[5, 1, 0], 0.52554786)
        self.assertAlmostEqual(aifg.data[6, 2, 0], 0.7433825)

    def test_load_ifg_mosaic_64(self):
        aifg = agilentMosaicIFG(DMT, MAT=False, dtype=float64)
        self.assertAlmostEqual(aifg.data[5, 1, 0], 0.7116039)



from agilent_format import agilentImage, agilentMosaic

ai = agilentImage("var/fpa_refl_vi.pb.seq", MAT=False)
am = agilentMosaic("var/2017-11-10 4X-25X/2017-11-10 4X-25X.dms", MAT=False)

for a in [ai, am]:
    assert a.data.shape[2] == len(a.info['wavenumbers'])
    assert a.wavenumbers == a.info['wavenumbers']
    assert a.width == a.data.shape[0]
    assert a.height == a.data.shape[1]
    assert a.filename in ["var/fpa_refl_vi.pb.bsp",
                          "var/2017-11-10 4X-25X/2017-11-10 4X-25X.dms" ]
    assert a.acqdate in ["Friday, November 10, 2017 15:22:29",
                         "Friday, November 10, 2017 13:46:28"]



import matplotlib.pyplot as plt
import matplotlib.image as mpimg
import numpy as np

from agilent_format import agilentImage, agilentMosaic

filename = "/data/users/Kelly/557-S2_1hrTFP_PT/557-S2_1hrTFP_PT.dms"
MAT = False

am = agilentMosaic(filename, MAT=MAT)
print(am.data.shape)
img = am.data.sum(axis=2)
print(img.shape)
if MAT:
    imgplot = plt.imshow(img, origin='upper')
else:
    imgplot = plt.imshow(img, origin='lower')
plt.show()



import matplotlib.pyplot as plt
import matplotlib.image as mpimg
import numpy as np

from agilent_format import agilentImage, agilentMosaic

am = agilentMosaic("/data/staff/reads/USAF 25X Mosaic/USAF 25X Mosaic.dms", MAT=False)
print(am.data.shape)
img = am.data.sum(axis=2)
extent = [0, am.data.shape[1]*am.info['FPA Pixel Size'],
          0, am.data.shape[0]*am.info['FPA Pixel Size']]
print(img.shape, extent)
imgplot = plt.imshow(img, origin='lower', extent=extent)
plt.show()

am = agilentMosaic("/data/staff/reads/USAF 25X Mosaic/USAF 25X Mosaic.dms", MAT=True)
print(am.data.shape)
img_m = am.data.sum(axis=2)
print(img.shape)
if np.array_equal(img_m, np.flipud(img)):
    print("MAT=True == MAT=False")
else:
    imgplot = plt.imshow(img)
    plt.show()



import unittest

from agilent_format import agilent

class TestUtils(unittest.TestCase):

    def test_fpa_size(self):
        Npts = 100
        datasize = 64**2 * Npts + 255
        self.assertEqual(agilent._fpa_size(datasize, Npts), 64)

    def test_fpa_size_nonsquare(self):
        Npts = 100
        datasize = 64 * 32 * Npts + 255
        with self.assertRaises(ValueError):
            agilent._fpa_size(datasize, Npts)
